@@ -54,6 +54,7 @@
 #define HPU  8 /* HP-UX */
 #define SGI  9 /* Irix */
 #define SOE 10 /* IBM's S/390 OpenEdition */
+#define TST 11 /* test code */
 
 /*
  * Try not to overflow the output buffer:
@@ -119,6 +120,8 @@
 #define PER_SANE_USER     0x0200
 #define PER_HPUX_x        0x0400
 #define PER_SVR4_x        0x0800
+#define PER_BSD_COLS      0x1000
+#define PER_UNIX_COLS     0x2000
 
 /* Simple selections by bit mask */
 #define SS_B_x 0x01
