@@ -14,6 +14,10 @@
 #include <string.h>
 #include <unistd.h>
 
+#if (__GNU_LIBRARY__ >= 6)
+# include <locale.h>
+#endif
+
 /* username lookups */
 #include <sys/types.h>
 #include <pwd.h>
@@ -187,36 +191,128 @@ static void check_headers(void){
   if(!head_normal) lines_to_next_header = -1; /* how UNIX does --noheader */
 }
 
-static unsigned needs_for_format;
-static unsigned needs_for_sort;
+/***** check sort needs */
+/* see what files need to be read, etc. */
+static unsigned check_sort_needs(sort_node *walk){
+  unsigned needs = 0;
+  while(walk){
+    needs |= walk->need;
+    walk = walk->next;
+  }
+  return needs;
+}
 
 /***** check needs */
 /* see what files need to be read, etc. */
-static void check_needs(void){
-  format_node *walk_pr = format_list;
-  sort_node   *walk_sr = sort_list;
-  /* selection doesn't currently have expensive needs */
-
-  while(walk_pr){
-    needs_for_format |= walk_pr->need;
-    walk_pr = walk_pr->next;
+static unsigned collect_format_needs(format_node *walk){
+  unsigned needs = 0;
+  while(walk){
+    needs |= walk->need;
+    walk = walk->next;
   }
-  if(bsd_e_option) needs_for_format |= PROC_FILLENV;
-
-  while(walk_sr){
-    needs_for_sort |= walk_sr->need;
-    walk_sr = walk_sr->next;
-  }
-
+  return needs;
 }
 
+static format_node *proc_format_list;
+static format_node *task_format_list;
+
+static unsigned needs_for_threads;
+static unsigned needs_for_sort;
+static unsigned proc_format_needs;
+static unsigned task_format_needs;
+
+#define needs_for_format (proc_format_needs|task_format_needs)
+
+#define PROC_ONLY_FLAGS (PROC_FILLENV|PROC_FILLARG|PROC_FILLCOM|PROC_FILLMEM)
+
+/***** munge lists and determine openproc() flags */
+static void lists_and_needs(void){
+  check_headers();
+
+  // only care about the difference when showing both
+  if(thread_flags & TF_show_both){
+    format_node pfn, tfn; // junk, to handle special case at begin of list
+    format_node *walk = format_list;
+    format_node *p_end = &pfn;
+    format_node *t_end = &tfn;
+    while(walk){
+      format_node *new = malloc(sizeof(format_node));
+      memcpy(new,walk,sizeof(format_node));
+      p_end->next = walk;
+      t_end->next = new;
+      p_end       = walk;
+      t_end       = new;
+      switch(walk->flags & CF_PRINT_MASK){
+      case CF_PRINT_THREAD_ONLY:
+        p_end->pr   = pr_nop;
+        p_end->need = 0;
+        break;
+      case CF_PRINT_PROCESS_ONLY:
+        t_end->pr   = pr_nop;
+        t_end->need = 0;
+        break;
+      default:
+        fprintf(stderr, "please report this bug\n");
+        // FALL THROUGH
+      case CF_PRINT_AS_NEEDED:
+      case CF_PRINT_EVERY_TIME:
+        break;
+      }
+      walk = walk->next;
+    }
+    t_end->next = NULL;
+    p_end->next = NULL;
+    proc_format_list = pfn.next;
+    task_format_list = tfn.next;
+  }else{
+    proc_format_list = format_list;
+    task_format_list = format_list;
+  }
+
+  proc_format_needs = collect_format_needs(proc_format_list);
+  task_format_needs = collect_format_needs(task_format_list);
+
+  needs_for_sort = check_sort_needs(sort_list);
+
+  // move process-only flags to the process
+  proc_format_needs |= (task_format_needs &~ PROC_ONLY_FLAGS);
+  task_format_needs &= ~PROC_ONLY_FLAGS;
+
+  if(bsd_c_option){
+    proc_format_needs &= ~PROC_FILLARG;
+    needs_for_sort    &= ~PROC_FILLARG;
+  }
+  if(!unix_f_option){
+    proc_format_needs &= ~PROC_FILLCOM;
+    needs_for_sort    &= ~PROC_FILLCOM;
+  }
+  // convert ARG to COM as a standard
+  if(proc_format_needs & PROC_FILLARG){
+    proc_format_needs |= PROC_FILLCOM;
+    proc_format_needs &= ~PROC_FILLARG;
+  }
+  if(bsd_e_option){
+    if(proc_format_needs&PROC_FILLCOM) proc_format_needs |= PROC_FILLENV;
+  }
+  
+  /* FIXME  broken filthy hack -- got to unify some stuff here */
+  if( ( (proc_format_needs|task_format_needs|needs_for_sort) & PROC_FILLWCHAN) && !wchan_is_number)
+    if (open_psdb(namelist_file)) wchan_is_number = 1;
+
+  if(thread_flags&TF_loose_tasks) needs_for_threads |= PROC_LOOSE_TASKS;
+}
+
+//////////////////////////////////////////////////////////////////////////
+
 /***** fill in %CPU; not in libproc because of include_dead_children */
 /* Note: for sorting, not display, so 0..0x7fffffff would be OK */
-static void fill_pcpu(proc_t *buf){
+static int want_this_proc_pcpu(proc_t *buf){
   unsigned long long used_jiffies;
   unsigned long pcpu = 0;
   unsigned long long avail_jiffies;
 
+  if(!want_this_proc(buf)) return 0;
+
   used_jiffies = buf->utime + buf->stime;
   if(include_dead_children) used_jiffies += (buf->cutime + buf->cstime);
 
@@ -224,44 +320,65 @@ static void fill_pcpu(proc_t *buf){
   if(avail_jiffies) pcpu = (used_jiffies << 24) / avail_jiffies;
 
   buf->pcpu = pcpu;  // fits in an int, summing children on 128 CPUs
-}
 
-/***** figure out what we need */
-static void compute_needs(void){
-  if(bsd_c_option){
-    needs_for_format &= ~PROC_FILLARG;
-    needs_for_sort   &= ~PROC_FILLARG;
-  }
-  if(!unix_f_option){
-    needs_for_format &= ~PROC_FILLCOM;
-    needs_for_sort   &= ~PROC_FILLCOM;
-  }
+  return 1;
 }
 
 /***** just display */
 static void simple_spew(void){
   proc_t buf;
   PROCTAB* ptp;
-  ptp = openproc(needs_for_format | needs_for_sort | needs_for_select);
+  ptp = openproc(needs_for_format | needs_for_sort | needs_for_select | needs_for_threads);
   if(!ptp) {
     fprintf(stderr, "Error: can not access /proc.\n");
     exit(1);
   }
-  if(!thread_flags) thread_flags=TF_show_proc;
   memset(&buf, '#', sizeof(proc_t));
-  while(readproc(ptp,&buf)){
-    if(want_this_proc(&buf)){
-      if(thread_flags & TF_show_proc) show_one_proc(&buf);
-      if(thread_flags & TF_show_task){
+  switch(thread_flags & (TF_show_proc|TF_loose_tasks|TF_show_task)){
+  case TF_show_proc:                   // normal non-thread output
+    while(readproc(ptp,&buf)){
+      if(want_this_proc(&buf)){
+        show_one_proc(&buf, proc_format_list);
+      }
+      if(buf.cmdline) free((void*)*buf.cmdline); // ought to reuse
+      if(buf.environ) free((void*)*buf.environ); // ought to reuse
+    }
+    break;
+  case TF_show_proc|TF_loose_tasks:    // H option
+    while(readproc(ptp,&buf)){
+      proc_t buf2;
+      // must still have the process allocated
+      while(readtask(ptp,&buf,&buf2)){
+        if(!want_this_proc(&buf)) continue;
+        show_one_proc(&buf2, task_format_list);
+      }
+      if(buf.cmdline) free((void*)*buf.cmdline); // ought to reuse
+      if(buf.environ) free((void*)*buf.environ); // ought to reuse
+    }
+    break;
+  case TF_show_proc|TF_show_task:      // m and -m options
+    while(readproc(ptp,&buf)){
+      if(want_this_proc(&buf)){
+        proc_t buf2;
+        show_one_proc(&buf, proc_format_list);
+        // must still have the process allocated
+        while(readtask(ptp,&buf,&buf2)) show_one_proc(&buf2, task_format_list);
+      }
+      if(buf.cmdline) free((void*)*buf.cmdline); // ought to reuse
+      if(buf.environ) free((void*)*buf.environ); // ought to reuse
+    }
+    break;
+  case TF_show_task:                   // -L and -T options
+    while(readproc(ptp,&buf)){
+      if(want_this_proc(&buf)){
         proc_t buf2;
         // must still have the process allocated
-        while(readtask(ptp,&buf,&buf2)) show_one_proc(&buf2);
-        // must not attempt to free cmdline and environ
+        while(readtask(ptp,&buf,&buf2)) show_one_proc(&buf2, task_format_list);
       }
+      if(buf.cmdline) free((void*)*buf.cmdline); // ought to reuse
+      if(buf.environ) free((void*)*buf.environ); // ought to reuse
     }
-    if(buf.cmdline) free((void*)*buf.cmdline); // ought to reuse
-    if(buf.environ) free((void*)*buf.environ); // ought to reuse
-//    memset(&buf, '#', sizeof(proc_t));
+    break;
   }
   closeproc(ptp);
 }
@@ -295,7 +412,8 @@ static void prep_forest_sort(void){
 }
 
 /* we rely on the POSIX requirement for zeroed memory */
-static proc_t *processes[98*1024];  // FIXME
+//static proc_t *processes[98*1024];  // FIXME
+static proc_t **processes;
 
 /***** compare function for qsort */
 static int compare_two_procs(const void *a, const void *b){
@@ -310,10 +428,16 @@ static int compare_two_procs(const void *a, const void *b){
 }
 
 /***** show pre-sorted array of process pointers */
-static void show_proc_array(int n){
+static void show_proc_array(PROCTAB *restrict ptp, int n){
   proc_t **p = processes;
   while(n--){
-    show_one_proc(*p);
+    if(thread_flags & TF_show_proc) show_one_proc(*p, proc_format_list);
+    if(thread_flags & TF_show_task){
+      proc_t buf2;
+      // must still have the process allocated
+      while(readtask(ptp,*p,&buf2)) show_one_proc(&buf2, task_format_list);
+      // must not attempt to free cmdline and environ
+    }
     /* no point freeing any of this -- won't need more mem */
 //    if((*p)->cmdline) free((void*)*(*p)->cmdline);
 //    if((*p)->environ) free((void*)*(*p)->environ);
@@ -334,7 +458,7 @@ static void show_tree(const int self, const int n, const int level, const int ha
     else             forest_prefix[level-1] = 'L';
     forest_prefix[level] = '\0';
   }
-  show_one_proc(processes[self]);  /* first show self */
+  show_one_proc(processes[self],format_list);  /* first show self */
   /* no point freeing any of this -- won't need more mem */
 //  if(processes[self]->cmdline) free((void*)*processes[self]->cmdline);
 //  if(processes[self]->environ) free((void*)*processes[self]->environ);
@@ -385,39 +509,46 @@ not_root:
   /* don't free the array because it takes time and ps will exit anyway */
 }
 
+static int want_this_proc_nop(proc_t *dummy){
+  (void)dummy;
+  return 1;
+}
+
 /***** sorted or forest */
 static void fancy_spew(void){
-  proc_t *retbuf = NULL;
+  proc_data_t *pd = NULL;
   PROCTAB *restrict ptp;
   int n = 0;  /* number of processes & index into array */
-  if(thread_flags){
-    fprintf(stderr, "can't have threads with sorting or forest output\n");
-    exit(49);
-  }
-  ptp = openproc(needs_for_format | needs_for_sort | needs_for_select);
+
+  ptp = openproc(needs_for_format | needs_for_sort | needs_for_select | needs_for_threads);
   if(!ptp) {
     fprintf(stderr, "Error: can not access /proc.\n");
     exit(1);
   }
-  while((retbuf = readproc(ptp,retbuf))){
-    if(want_this_proc(retbuf)){
-      fill_pcpu(retbuf); // in case we might sort by %cpu
-      processes[n++] = retbuf;
-      retbuf = NULL;     // NULL asks readproc to allocate
-    }
+
+  if(thread_flags & TF_loose_tasks){
+    pd = readproctab2(want_this_proc_nop, want_this_proc_pcpu, ptp);
+  }else{
+    pd = readproctab2(want_this_proc_pcpu, (void*)0xdeadbeaful, ptp);
   }
-  if(retbuf) free(retbuf);
-  closeproc(ptp);
+  n = pd->n;
+  processes = pd->tab;
+
   if(!n) return;  /* no processes */
   if(forest_type) prep_forest_sort();
   qsort(processes, n, sizeof(proc_t*), compare_two_procs);
   if(forest_type) show_forest(n);
-  else show_proc_array(n);
+  else show_proc_array(ptp,n);
+  closeproc(ptp);
 }
 
 
 /***** no comment */
 int main(int argc, char *argv[]){
+#if (__GNU_LIBRARY__ >= 6)
+  setlocale (LC_CTYPE, "");
+#endif
+
 #ifdef DEBUG
   init_stack_trace(argv[0]);
 #else
@@ -451,16 +582,11 @@ int main(int argc, char *argv[]){
   trace("======= ps output follows =======\n");
 
   init_output(); /* must be between parser and output */
-  check_headers();
 
-  check_needs();
-  /* filthy hack -- got to unify some stuff here */
-  if( ( (needs_for_format|needs_for_sort) & PROC_FILLWCHAN) && !wchan_is_number)
-    if (open_psdb(namelist_file)) wchan_is_number = 1;
-  compute_needs();
+  lists_and_needs();
 
   if(forest_type || sort_list) fancy_spew(); /* sort or forest */
   else simple_spew(); /* no sort, no forest */
-  show_one_proc((proc_t *)-1); /* no output yet? */
+  show_one_proc((proc_t *)-1,format_list); /* no output yet? */
   return 0;
 }
